import java.io.IOException;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EEnum;
import org.eclipse.emf.ecore.EEnumLiteral;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.impl.EObjectImpl;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;

public class Ecore2Clafer extends EObjectImpl {
	
	public static void main(String[] args) {
		try {
			new Ecore2Clafer().translate(args[0]);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}	  

	public void translate(String filepath) throws IOException {
		URI fileURI = URI.createFileURI(filepath);
		Resource resource = new XMIResourceFactoryImpl().createResource(fileURI);
		resource.load(null);
		
		for (EObject eobject : resource.getContents())
		{
			EPackage epackage = (EPackage)eobject;
			
			// package (Clafer: module) name
			//System.out.println(formatName(epackage.getName()));
			
			for (EClassifier eclassifier : epackage.getEClassifiers())
			{
			    System.out.print(translateEClassifier(eclassifier));
			}
		}
	}

	public String translateEClassifier(EClassifier eclassifier)
	{
	    StringBuilder sb = new StringBuilder();
	    
	    
        
        if (eclassifier instanceof EClass)
        {
            sb.append("abstract " + formatName(eclassifier.getName()) + 
                    translateEClass((EClass) eclassifier));
        }
        else if (eclassifier instanceof EEnum)
        {
            sb.append("enum " + formatName(eclassifier.getName()) + " = " + translateEEnum((EEnum) eclassifier) + "\n");
        }
        else
        {
            sb.append("abstract " + formatName(eclassifier.getName()) + "\n");
        }
        sb.append("\n");
        return sb.toString();
	}
	
	public String translateEClass(EClass eClass)
	{
	    StringBuilder sb = new StringBuilder();
	    
	    sb.append(translateESuperTypes(eClass) + "\n");
	    
        for (EAttribute eAttribute : eClass.getEAttributes())
        {
            sb.append(translateEAttribute(eAttribute));
        }
        
        for (EReference eReference : eClass.getEReferences())
        {
            sb.append(translateEReference(eReference));
        }
        return sb.toString();
	}
	
	public String translateEEnum(EEnum eEnum)
	{
	    StringBuilder sb = new StringBuilder();
	    
	    for (EEnumLiteral eEnumLiteral : eEnum.getELiterals())
	    {
	        sb.append(" | " +  eEnumLiteral.getName());
	    }
	    
	    return sb.substring(3);
	}
	
	public String translateESuperTypes(EClass eClass)
	{
	    StringBuilder sb = new StringBuilder();
	    
	    EList<EClass> eSuperTypes = eClass.getESuperTypes();
        
        if (!eSuperTypes.isEmpty())
        {
            sb.append(" extends ");
            for (EClass ec : eSuperTypes)
            {
                sb.append(formatName(ec.getName()) + " ");    
            }
        }
        return sb.toString();
	}
	
	public String translateEAttribute(EAttribute eAttribute)
	{
        StringBuilder sb = new StringBuilder();
        
        sb.append(indent(1) + formatName(eAttribute.getName()) + 
                (eAttribute.getEAttributeType() instanceof EEnum ? " -> " : " : ") +
                formatName(eAttribute.getEAttributeType().getName()) + " " +
                formatCard(eAttribute.getLowerBound(),
                           eAttribute.getUpperBound()) + "\n");
        return sb.toString();
	}

	public String translateEReference(EReference eReference)
	{

        StringBuilder sb = new StringBuilder();
        
        sb.append(indent(1) + formatName(eReference.getName()) + " " +
            (eReference.isContainment() ? ":" : "->") + " " +
            formatName(eReference.getEReferenceType().getName()) + " " +
            formatCard(eReference.getLowerBound(), eReference.getUpperBound()) + "\n");
        
        if (null != eReference.getEOpposite())
        {
            sb.append(indent(1) + (eReference.isContainment() ? indent(1) + 
                    "[parent in " : "[all x : this." + formatName(eReference.getName()) + " | this in x.") +
                    formatName(eReference.getEOpposite().getName()) +"]\n");
        }
        return sb.toString();
	}
	
	public String indent(int n)
	{
	    StringBuilder sb = new StringBuilder();
	    
	    for (int i = 0; i < n; ++i)
	    {
	        sb.append("    ");
	    }
	    return sb.toString();
	}
	
	public String formatCard(int lowerBound, int upperBound)
	{
	    if (0 == lowerBound && 1 == upperBound)
	        return "?";
	    else if (0 == lowerBound && -1 == upperBound)
	        return "*";
	    else if (1 == lowerBound && -1 == upperBound)
	        return "+";
	    else if (1 == lowerBound && 1 == upperBound)
	        return "";
	    else
	        return lowerBound + ".." + (-1 == upperBound ? "*" : upperBound);
	}
	
	public String formatName(String name)
	{
	    return name.replace("_", "");
	}
	
}
